Imports System.Runtime.InteropServices
Imports System.ComponentModel
Imports System.Windows.Forms

'Add-in Express Add-in Module
<GuidAttribute("D3703BAA-9A1B-46CF-8E89-4AEA49C51A92"), ProgIdAttribute("OLItemsExample.AddinModule"), RunInstaller(True)> _
Public Class AddinModule
    Inherits AddinExpress.MSO.ADXAddinModule
    Private MyItems As OlItems_SinkHelper

#Region " Component Designer generated code. "
    'Required by designer
    Private components As System.ComponentModel.IContainer

    'Required by designer - do not modify
    'the following method
    Private Sub InitializeComponent()
        '
        'AddinModule
        '
        Me.AddinName = "Outlook Item Events"
        Me.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook

    End Sub

#End Region

#Region " ADX automatic code "

    'Required by Add-in Express - do not modify
    'the methods within this region

    Public Overrides Function GetContainer() As System.ComponentModel.IContainer
        GetContainer = components
    End Function

    <ComRegisterFunctionAttribute()> _
    Public Shared Sub AddinRegister(ByVal t As Type)
        AddinExpress.MSO.ADXAddinModule.ADXRegister(t)
    End Sub

    <ComUnregisterFunctionAttribute()> _
    Public Shared Sub AddinUnregister(ByVal t As Type)
        AddinExpress.MSO.ADXAddinModule.ADXUnregister(t)
    End Sub

    Public Overrides Sub UninstallControls()
        MyBase.UninstallControls()
    End Sub

#End Region

#Region " OlItems_SinkHelper "

    <ComImport(), Guid("00063077-0000-0000-C000-000000000046"), InterfaceType(CType(2, Short))> _
    Public Interface IFolderItemsEvents
        ' Methods
        <PreserveSig(), DispId(61441)> _
        Sub ItemAdd(<MarshalAs(UnmanagedType.IDispatch)> ByVal Item As Object)
        <PreserveSig(), DispId(61442)> _
        Sub ItemChange(<MarshalAs(UnmanagedType.IDispatch)> ByVal Item As Object)
        <PreserveSig(), DispId(61443)> _
        Sub ItemRemove()
    End Interface

    <GuidAttribute("3BFF853D-22EC-47c7-9A29-758D3EC9D5F9"), ClassInterface(ClassInterfaceType.None)> _
    Public Class OlItems_SinkHelper
        Implements IFolderItemsEvents
        Implements System.IDisposable

        Private Disposed As Boolean = False
        Private M_oConnectionPoint As UCOMIConnectionPoint
        Private M_Cookie As Integer
        Private Owner As AddinModule
        Private OutlookItems As Outlook.Items

        Public Sub New(ByRef connect As AddinModule)
            M_oConnectionPoint = Nothing
            M_Cookie = 0
            Owner = connect
        End Sub

        Overloads Sub Finalize()
            FreeState()
            MyBase.Finalize()
        End Sub

        Public Sub Dispose() Implements IDisposable.Dispose
            FreeState()
            GC.SuppressFinalize(Me)
        End Sub

        Private Sub FreeState()
            If Not Disposed Then
                RemoveConnection()
                Owner = Nothing
                Disposed = True
            End If
        End Sub

        Public Function ConnectTo(ByRef items As Outlook.Items) As Boolean
            If M_Cookie <> 0 Then Return False
            Try
                If Not (items Is Nothing) Then
                    OutlookItems = items
                    Dim myGuid As Guid = New Guid("{00063077-0000-0000-C000-000000000046}")
                    Dim oConnPointContainer As UCOMIConnectionPointContainer = items
                    oConnPointContainer.FindConnectionPoint(myGuid, M_oConnectionPoint)
                    If Not (M_oConnectionPoint Is Nothing) Then
                        M_oConnectionPoint.Advise(Me, M_Cookie)
                    End If
                    oConnPointContainer = Nothing
                End If
            Catch
                Return False
            End Try
            Return True
        End Function

        Public Sub RemoveConnection()
            If M_Cookie <> 0 And Not (M_oConnectionPoint Is Nothing) Then
                Try
                    M_oConnectionPoint.Unadvise(M_Cookie)
                Catch
                End Try
                Marshal.ReleaseComObject(M_oConnectionPoint)
                M_Cookie = 0
            End If
            If Not (OutlookItems Is Nothing) Then
                Marshal.ReleaseComObject(OutlookItems)
                OutlookItems = Nothing
            End If
        End Sub

        Public Sub ItemAdd(ByVal Item As Object) Implements IFolderItemsEvents.ItemAdd

            MessageBox.Show("Item added")
            Marshal.ReleaseComObject(Item)

        End Sub

        Public Sub ItemChange(ByVal Item As Object) Implements IFolderItemsEvents.ItemChange

            MessageBox.Show("Item changed")
            Marshal.ReleaseComObject(Item)

        End Sub

        Public Sub ItemRemove() Implements IFolderItemsEvents.ItemRemove

            MessageBox.Show("Item removed")

        End Sub

    End Class
#End Region

    'Add-in Module Implementation

    Public Sub New()
        MyBase.New()

        'This call is required by the Component Designer
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    Private Function OutlookHost() As Outlook.Application
        OutlookHost = Me.HostApplication
    End Function

    Private Sub AddinModule_AddinStartupComplete(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.AddinStartupComplete
        Dim activeExplorer As Outlook._Explorer
        Dim currentFolder As Outlook.MAPIFolder

        activeExplorer = OutlookHost().ActiveExplorer()
        currentFolder = activeExplorer.CurrentFolder
        MyItems = New OlItems_SinkHelper(Me)
        If Not (currentFolder Is Nothing) Then
            MyItems.ConnectTo(currentFolder.Items)
        End If
        Marshal.ReleaseComObject(currentFolder)
        Marshal.ReleaseComObject(activeExplorer)

    End Sub

    Private Sub AddinModule_AddinFinalize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.AddinFinalize

        If Not (MyItems Is Nothing) Then
            MyItems.Dispose()
        End If

    End Sub

    Private Sub AddinModule_OlExplorerFolderSwitch(ByVal sender As Object, ByVal explorer As Object) Handles MyBase.OlExplorerFolderSwitch
        Dim expl As Outlook.Explorer = explorer
        Dim Folder As Outlook.MAPIFolder = expl.CurrentFolder

        If Not (MyItems Is Nothing) Then
            MyItems.RemoveConnection()
            MyItems.ConnectTo(Folder.Items)
        End If
        Marshal.ReleaseComObject(Folder)

    End Sub
End Class

